'From Squeak3.4 of 1 March 2003 [latest update: #5170] on 17 September 2003 at 7:03:18 pm'!
Dictionary variableSubclass: #DictionaryWithDefault
	instanceVariableNames: 'defaultBlock '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Unordered'!

!DictionaryWithDefault commentStamp: '<historical>' prior: 0!
A DictionaryWithDefault is like a Dictionary, except that it is initialized with a value (or block) that is (yields) the default value for any key.

A subsequent access to a key that has not been explicitly inserted into the dictionary will yield the default value (calculated by executing the block), and insert the (key,default value) association into the dictionary.!

IdentityDictionary variableSubclass: #IdentityDictionaryWithDefault
	instanceVariableNames: 'defaultBlock '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Unordered'!

!IdentityDictionaryWithDefault commentStamp: '<historical>' prior: 0!
An IdentityDictionaryWithDefault is like an IdentityDictionary, except that it is initialized with a value (or block) that is (yields) the default value for any key.

A subsequent access to a key that has not been explicitly inserted into the dictionary will yield the default value (calculated by executing the block), and insert the (key,default value) association into the dictionary.

See DictionaryWithDefault for examples.!

Set variableSubclass: #KeyedSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Unordered'!

!KeyedSet commentStamp: '<historical>' prior: 0!
A KeyedSet has the ability to locate an element of the set based on its hash, using at:.!


!Collection methodsFor: 'accessing' stamp: 'len 2/18/2003 21:40'!
anyIfNone: aBlock
	"Answer a representative sample of the receiver. This method can
	be helpful when needing to preinfer the nature of the contents of 
	semi-homogeneous collections."

	self do: [:each | ^ each].
	^aBlock value! !

!Collection methodsFor: 'copying' stamp: 'len 2/13/2003 23:19'!
copyEmpty
	^ self species new! !

!Collection methodsFor: 'copying' stamp: 'len 2/13/2003 23:19'!
copyEmpty: anInteger
	^ self species new: anInteger! !


!Bag methodsFor: 'removing'!
remove: oldObject howMany: count
	"Remove count occurrences of oldObject from the receiver.  
	Answer the number of remaining elements."

	^self remove: oldObject howMany: count ifAbsent: [self notFoundError]! !

!Bag methodsFor: 'removing'!
remove: oldObject howMany: count ifAbsent: exceptionBlock
	"Remove count occurrences of oldObject from the receiver.  Answer the number of remaining elements.  If the receiver contains less than count occurrences of oldObject, evalaute the exception block."

	^(self includes: oldObject)
		ifTrue: [| currentCount |
				(currentCount := contents at: oldObject) < count
					ifTrue: [exceptionBlock value]
					ifFalse: [currentCount = count
								ifTrue: [contents removeKey: oldObject.
										0]
								ifFalse: [currentCount := currentCount - count.
										contents at: oldObject put: currentCount.
										currentCount]]]
		ifFalse: [exceptionBlock value]! !

!Bag methodsFor: 'removing'!
removeAllOccurrences: obj ifNone: aBlock
	"Remove all occurrences of obj; evaluate aBlock if there were none."
	^contents removeKey: obj ifAbsent: aBlock! !

!Bag methodsFor: 'removing'!
removeOne: oldObject
	"Remove oldObject as one of the receiver's elements.  If several of the elements are equal to oldObject, only one is removed.  Answer the number of remaining elements."

	^self remove: oldObject howMany: 1! !

!Bag methodsFor: 'removing'!
removeOne: oldObject ifAbsent: exceptionBlock
	"Remove oldObject as one of the receiver's elements.  If several of the elements are equal to oldObject, only one is removed.  Answer the number of remaining elements.  If the receiver does not contain oldObject, evalaute the exception block."

	^self remove: oldObject howMany: 1 ifAbsent: exceptionBlock! !

!Bag methodsFor: 'frequencies'!
frequencyDistribution
	"Return a dictionary mapping each different thing in the bag to its percentage of the total."
	| total d |
	total := contents inject: 0 into: [ :subTotal :next | subTotal + next].
	d := Dictionary new.
	contents associationsDo: [ :assoc |
		d at: assoc key put: (assoc value/total) asFloat].
	^d! !


!Collection class methodsFor: 'instance creation'!
accumulate: iteratorBlock
	"Return a new instance of myself by evaluating the iteratorBlock.
	Examples:
		| x oc set |
		x := #( 1 2 3 4 1 ).
		oc := OrderedCollection accumulate: [ :incBlock | x do: incBlock].
		Transcript cr; show: oc printString.
		set := Set accumulate: [ :incBlock | x do: incBlock].
		Transcript cr; show: set printString"

	| collection | 
	collection := self new.
	iteratorBlock value: [ :element | collection add: element].
	^collection! !

!Collection class methodsFor: 'instance creation'!
accumulate: iteratorSymbol on: collection
	"Return a new instance of myself by sending the iteratorSymbol to the collection.
	Examples:
		| x oc set |
		x := #( 1 2 3 4 1 ).
		oc := OrderedCollection accumulate: #do: on: x.
		Transcript cr; show: oc printString.
		set := Set accumulate: #do: on: x.
		Transcript cr; show: set printString"

	| newCollection | 
	newCollection := self new.
	collection perform: iteratorSymbol with: [ :element | newCollection add: element].
	^newCollection! !


!DictionaryWithDefault methodsFor: 'copying'!
copyEmpty: aSize
	^(super copyEmpty: aSize) defaultBlock: defaultBlock! !

!DictionaryWithDefault methodsFor: 'accessing'!
associationAt: key 
	"Answer the association at key."

	^super associationAt: key ifAbsent:
		[| value |
		value := defaultBlock value: key.
		super at: key put: value.
		super associationAt: key]! !

!DictionaryWithDefault methodsFor: 'accessing'!
at: key 
	"Answer the value at key."

	^super at: key ifAbsent:
		[| value |
		value := defaultBlock value: key.
		super at: key put: value.
		super at: key]! !

!DictionaryWithDefault methodsFor: 'initialization'!
defaultBlock: aBlock
	defaultBlock := aBlock! !


!DictionaryWithDefault class methodsFor: 'instance creation'!
newIdentity
	"Default map is identity."
	^self newWithDefaultValueBlock: [ :x | x]! !

!DictionaryWithDefault class methodsFor: 'instance creation'!
newWithDefaultValue: aValue
	^super new defaultBlock: [ :ignored | aValue]! !

!DictionaryWithDefault class methodsFor: 'instance creation'!
newWithDefaultValueBlock: aBlock
	^super new defaultBlock: aBlock! !

!DictionaryWithDefault class methodsFor: 'examples'!
example
	"DictionaryWithDefault example"
	"| d |
	d := DictionaryWithDefault example.
	d at: #foo put: 2.
	Transcript show: (d at: #foo) printString ; space ; show: (d at: #bar) printString"
	"A dictionary of symbols whose default value is the number of characters in the symbol."
	^DictionaryWithDefault newWithDefaultValueBlock: [ :key | key size]! !

!DictionaryWithDefault class methodsFor: 'examples'!
example2
	"DictionaryWithDefault example2"
	"| d |
	d := DictionaryWithDefault example2.
	#(1 2 3 5 7 11 13 17 19) do: [ :n | d at: n put: 1].
	1 to: 20 do: [ :n | Transcript show: (d at: n) printString ; space]"
	"A dictionary of whose default value is 0."
	^DictionaryWithDefault newWithDefaultValue: 0! !


!IdentityDictionaryWithDefault methodsFor: 'copying'!
copyEmpty: aSize
	^(super copyEmpty: aSize) defaultBlock: defaultBlock! !

!IdentityDictionaryWithDefault methodsFor: 'accessing'!
associationAt: key 
	"Answer the association at key."

	^super associationAt: key ifAbsent:
		[| value |
		value := defaultBlock value: key.
		super at: key put: value.
		super associationAt: key]! !

!IdentityDictionaryWithDefault methodsFor: 'accessing'!
at: key 
	"Answer the value at key."

	^super at: key ifAbsent:
		[| value |
		value := defaultBlock value: key.
		super at: key put: value.
		super at: key]! !

!IdentityDictionaryWithDefault methodsFor: 'initialization'!
defaultBlock: aBlock
	defaultBlock := aBlock! !


!IdentityDictionaryWithDefault class methodsFor: 'instance creation'!
newIdentity
	"Default map is identity."
	^self newWithDefaultValueBlock: [ :x | x]! !

!IdentityDictionaryWithDefault class methodsFor: 'instance creation'!
newWithDefaultValue: aValue
	^super new defaultBlock: [ :ignored | aValue]! !

!IdentityDictionaryWithDefault class methodsFor: 'instance creation'!
newWithDefaultValueBlock: aBlock
	^super new defaultBlock: aBlock! !


!KeyedSet methodsFor: 'accessing' stamp: 'len 2/12/2003 16:29'!
at: key
	^self at: key ifAbsent: [self error: 'key not found']! !

!KeyedSet methodsFor: 'accessing' stamp: 'len 2/12/2003 16:45'!
at: key ifAbsent: aBlock
	| index |
	^ (index _ self scanFor: key) = 0
		ifTrue: [^ aBlock value]
		ifFalse: [(array at: index) ifNil: [aBlock value]]! !


!Collection reorganize!
('accessing' anyIfNone: anyOne atRandom capacity size)
('adapting' adaptToCollection:andSend: adaptToNumber:andSend: adaptToPoint:andSend: adaptToString:andSend:)
('adding' add: add:withOccurrences: addAll: addIfNotPresent:)
('arithmetic' * + - / // \\)
('comparing' hash)
('converting' asArray asBag asByteArray asCharacterSet asIdentitySet asIdentitySkipList asOrderedCollection asSet asSkipList asSkipList: asSortedArray asSortedCollection asSortedCollection: topologicallySortedUsing:)
('copying' copyEmpty copyEmpty: copyWith: copyWithDependent: copyWithout: copyWithoutAll:)
('enumerating' allSatisfy: anySatisfy: associationsDo: collect: collect:thenSelect: count: detect: detect:ifNone: detectMax: detectMin: detectSum: difference: do: do:separatedBy: do:without: groupBy:having: inject:into: intersection: noneSatisfy: reject: select: select:thenCollect: union:)
('filter streaming' contents flattenOnStream: write:)
('math functions' abs average ceiling floor log max median min negated range reciprocal rounded sqrt squared sum truncated)
('printing' printElementsOn: printNameOn: printOn: storeOn:)
('private' emptyCheck errorEmptyCollection errorNoMatch errorNotFound: errorNotKeyed toBraceStack:)
('removing' remove: remove:ifAbsent: removeAll: removeAllFoundIn: removeAllSuchThat:)
('testing' identityIncludes: includes: includesAllOf: includesAnyOf: includesSubstringAnywhere: isCollection isEmpty isEmptyOrNil isSequenceable occurrencesOf:)
('*packageinfo' gather:)
('*sbcompat-VWCompatibility')
('*sb-operations' itemActionFor:)
('*Refactory-RBAddonsReasonable' , contains:)
('*Refactory-RBIfEmptyIfNotEmpty' ifEmpty: ifEmpty:ifNotEmpty: ifNotEmpty: ifNotEmpty:ifEmpty: notEmpty)
!

